# Generated from ObjectiveCParser.g4 by ANTLR 4.7.1
from QBCodeSpecs.antlr4 import *

# This class defines a complete listener for a parse tree produced by ObjectiveCParser.
class ObjectiveCParserListener(ParseTreeListener):

    # Enter a parse tree produced by ObjectiveCParser#translationUnit.
    def enterTranslationUnit(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#translationUnit.
    def exitTranslationUnit(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#topLevelDeclaration.
    def enterTopLevelDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#topLevelDeclaration.
    def exitTopLevelDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#importDeclaration.
    def enterImportDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#importDeclaration.
    def exitImportDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classInterface.
    def enterClassInterface(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classInterface.
    def exitClassInterface(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#categoryInterface.
    def enterCategoryInterface(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#categoryInterface.
    def exitCategoryInterface(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classImplementation.
    def enterClassImplementation(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classImplementation.
    def exitClassImplementation(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#categoryImplementation.
    def enterCategoryImplementation(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#categoryImplementation.
    def exitCategoryImplementation(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#genericTypeSpecifier.
    def enterGenericTypeSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#genericTypeSpecifier.
    def exitGenericTypeSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolDeclaration.
    def enterProtocolDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolDeclaration.
    def exitProtocolDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolDeclarationSection.
    def enterProtocolDeclarationSection(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolDeclarationSection.
    def exitProtocolDeclarationSection(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolDeclarationList.
    def enterProtocolDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolDeclarationList.
    def exitProtocolDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classDeclarationList.
    def enterClassDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classDeclarationList.
    def exitClassDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolList.
    def enterProtocolList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolList.
    def exitProtocolList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyDeclaration.
    def enterPropertyDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyDeclaration.
    def exitPropertyDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyAttributesList.
    def enterPropertyAttributesList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyAttributesList.
    def exitPropertyAttributesList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyAttribute.
    def enterPropertyAttribute(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyAttribute.
    def exitPropertyAttribute(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolName.
    def enterProtocolName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolName.
    def exitProtocolName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#instanceVariables.
    def enterInstanceVariables(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#instanceVariables.
    def exitInstanceVariables(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#visibilitySection.
    def enterVisibilitySection(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#visibilitySection.
    def exitVisibilitySection(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#accessModifier.
    def enterAccessModifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#accessModifier.
    def exitAccessModifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#interfaceDeclarationList.
    def enterInterfaceDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#interfaceDeclarationList.
    def exitInterfaceDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classMethodDeclaration.
    def enterClassMethodDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classMethodDeclaration.
    def exitClassMethodDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#instanceMethodDeclaration.
    def enterInstanceMethodDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#instanceMethodDeclaration.
    def exitInstanceMethodDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodDeclaration.
    def enterMethodDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodDeclaration.
    def exitMethodDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#implementationDefinitionList.
    def enterImplementationDefinitionList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#implementationDefinitionList.
    def exitImplementationDefinitionList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#classMethodDefinition.
    def enterClassMethodDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#classMethodDefinition.
    def exitClassMethodDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#instanceMethodDefinition.
    def enterInstanceMethodDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#instanceMethodDefinition.
    def exitInstanceMethodDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodDefinition.
    def enterMethodDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodDefinition.
    def exitMethodDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodSelector.
    def enterMethodSelector(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodSelector.
    def exitMethodSelector(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#keywordDeclarator.
    def enterKeywordDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#keywordDeclarator.
    def exitKeywordDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selector.
    def enterSelector(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selector.
    def exitSelector(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#methodType.
    def enterMethodType(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#methodType.
    def exitMethodType(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertyImplementation.
    def enterPropertyImplementation(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertyImplementation.
    def exitPropertyImplementation(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertySynthesizeList.
    def enterPropertySynthesizeList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertySynthesizeList.
    def exitPropertySynthesizeList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#propertySynthesizeItem.
    def enterPropertySynthesizeItem(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#propertySynthesizeItem.
    def exitPropertySynthesizeItem(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#blockType.
    def enterBlockType(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#blockType.
    def exitBlockType(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#genericsSpecifier.
    def enterGenericsSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#genericsSpecifier.
    def exitGenericsSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeSpecifierWithPrefixes.
    def enterTypeSpecifierWithPrefixes(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeSpecifierWithPrefixes.
    def exitTypeSpecifierWithPrefixes(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#dictionaryExpression.
    def enterDictionaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#dictionaryExpression.
    def exitDictionaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#dictionaryPair.
    def enterDictionaryPair(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#dictionaryPair.
    def exitDictionaryPair(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#arrayExpression.
    def enterArrayExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#arrayExpression.
    def exitArrayExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#boxExpression.
    def enterBoxExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#boxExpression.
    def exitBoxExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#blockParameters.
    def enterBlockParameters(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#blockParameters.
    def exitBlockParameters(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeVariableDeclaratorOrName.
    def enterTypeVariableDeclaratorOrName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeVariableDeclaratorOrName.
    def exitTypeVariableDeclaratorOrName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#blockExpression.
    def enterBlockExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#blockExpression.
    def exitBlockExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#messageExpression.
    def enterMessageExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#messageExpression.
    def exitMessageExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#receiver.
    def enterReceiver(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#receiver.
    def exitReceiver(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#messageSelector.
    def enterMessageSelector(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#messageSelector.
    def exitMessageSelector(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#keywordArgument.
    def enterKeywordArgument(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#keywordArgument.
    def exitKeywordArgument(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#keywordArgumentType.
    def enterKeywordArgumentType(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#keywordArgumentType.
    def exitKeywordArgumentType(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selectorExpression.
    def enterSelectorExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selectorExpression.
    def exitSelectorExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selectorName.
    def enterSelectorName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selectorName.
    def exitSelectorName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolExpression.
    def enterProtocolExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolExpression.
    def exitProtocolExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#encodeExpression.
    def enterEncodeExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#encodeExpression.
    def exitEncodeExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeVariableDeclarator.
    def enterTypeVariableDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeVariableDeclarator.
    def exitTypeVariableDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#throwStatement.
    def enterThrowStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#throwStatement.
    def exitThrowStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#tryBlock.
    def enterTryBlock(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#tryBlock.
    def exitTryBlock(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#catchStatement.
    def enterCatchStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#catchStatement.
    def exitCatchStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#synchronizedStatement.
    def enterSynchronizedStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#synchronizedStatement.
    def exitSynchronizedStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#autoreleaseStatement.
    def enterAutoreleaseStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#autoreleaseStatement.
    def exitAutoreleaseStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionDeclaration.
    def enterFunctionDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionDeclaration.
    def exitFunctionDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionDefinition.
    def enterFunctionDefinition(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionDefinition.
    def exitFunctionDefinition(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionSignature.
    def enterFunctionSignature(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionSignature.
    def exitFunctionSignature(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attribute.
    def enterAttribute(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attribute.
    def exitAttribute(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeName.
    def enterAttributeName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeName.
    def exitAttributeName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameters.
    def enterAttributeParameters(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameters.
    def exitAttributeParameters(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameterList.
    def enterAttributeParameterList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameterList.
    def exitAttributeParameterList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameter.
    def enterAttributeParameter(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameter.
    def exitAttributeParameter(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeParameterAssignment.
    def enterAttributeParameterAssignment(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeParameterAssignment.
    def exitAttributeParameterAssignment(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declaration.
    def enterDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declaration.
    def exitDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#functionCallExpression.
    def enterFunctionCallExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#functionCallExpression.
    def exitFunctionCallExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumDeclaration.
    def enterEnumDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumDeclaration.
    def exitEnumDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#varDeclaration.
    def enterVarDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#varDeclaration.
    def exitVarDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typedefDeclaration.
    def enterTypedefDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typedefDeclaration.
    def exitTypedefDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeDeclaratorList.
    def enterTypeDeclaratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeDeclaratorList.
    def exitTypeDeclaratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeDeclarator.
    def enterTypeDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeDeclarator.
    def exitTypeDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declarationSpecifiers.
    def enterDeclarationSpecifiers(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declarationSpecifiers.
    def exitDeclarationSpecifiers(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#attributeSpecifier.
    def enterAttributeSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#attributeSpecifier.
    def exitAttributeSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initDeclaratorList.
    def enterInitDeclaratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initDeclaratorList.
    def exitInitDeclaratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initDeclarator.
    def enterInitDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initDeclarator.
    def exitInitDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#structOrUnionSpecifier.
    def enterStructOrUnionSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#structOrUnionSpecifier.
    def exitStructOrUnionSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#fieldDeclaration.
    def enterFieldDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#fieldDeclaration.
    def exitFieldDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#specifierQualifierList.
    def enterSpecifierQualifierList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#specifierQualifierList.
    def exitSpecifierQualifierList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#ibOutletQualifier.
    def enterIbOutletQualifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#ibOutletQualifier.
    def exitIbOutletQualifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#arcBehaviourSpecifier.
    def enterArcBehaviourSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#arcBehaviourSpecifier.
    def exitArcBehaviourSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#nullabilitySpecifier.
    def enterNullabilitySpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#nullabilitySpecifier.
    def exitNullabilitySpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#storageClassSpecifier.
    def enterStorageClassSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#storageClassSpecifier.
    def exitStorageClassSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typePrefix.
    def enterTypePrefix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typePrefix.
    def exitTypePrefix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeQualifier.
    def enterTypeQualifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeQualifier.
    def exitTypeQualifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#protocolQualifier.
    def enterProtocolQualifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#protocolQualifier.
    def exitProtocolQualifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeSpecifier.
    def enterTypeSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeSpecifier.
    def exitTypeSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeofExpression.
    def enterTypeofExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeofExpression.
    def exitTypeofExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#fieldDeclaratorList.
    def enterFieldDeclaratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#fieldDeclaratorList.
    def exitFieldDeclaratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#fieldDeclarator.
    def enterFieldDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#fieldDeclarator.
    def exitFieldDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumSpecifier.
    def enterEnumSpecifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumSpecifier.
    def exitEnumSpecifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumeratorList.
    def enterEnumeratorList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumeratorList.
    def exitEnumeratorList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumerator.
    def enterEnumerator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumerator.
    def exitEnumerator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#enumeratorIdentifier.
    def enterEnumeratorIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#enumeratorIdentifier.
    def exitEnumeratorIdentifier(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#directDeclarator.
    def enterDirectDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#directDeclarator.
    def exitDirectDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declaratorSuffix.
    def enterDeclaratorSuffix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declaratorSuffix.
    def exitDeclaratorSuffix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#parameterList.
    def enterParameterList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#parameterList.
    def exitParameterList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#pointer.
    def enterPointer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#pointer.
    def exitPointer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#macro.
    def enterMacro(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#macro.
    def exitMacro(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#arrayInitializer.
    def enterArrayInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#arrayInitializer.
    def exitArrayInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#structInitializer.
    def enterStructInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#structInitializer.
    def exitStructInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initializerList.
    def enterInitializerList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initializerList.
    def exitInitializerList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#typeName.
    def enterTypeName(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#typeName.
    def exitTypeName(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#abstractDeclarator.
    def enterAbstractDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#abstractDeclarator.
    def exitAbstractDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#abstractDeclaratorSuffix.
    def enterAbstractDeclaratorSuffix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#abstractDeclaratorSuffix.
    def exitAbstractDeclaratorSuffix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#parameterDeclarationList.
    def enterParameterDeclarationList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#parameterDeclarationList.
    def exitParameterDeclarationList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#parameterDeclaration.
    def enterParameterDeclaration(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#parameterDeclaration.
    def exitParameterDeclaration(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#declarator.
    def enterDeclarator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#declarator.
    def exitDeclarator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#statement.
    def enterStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#statement.
    def exitStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#labeledStatement.
    def enterLabeledStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#labeledStatement.
    def exitLabeledStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#rangeExpression.
    def enterRangeExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#rangeExpression.
    def exitRangeExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#compoundStatement.
    def enterCompoundStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#compoundStatement.
    def exitCompoundStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#selectionStatement.
    def enterSelectionStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#selectionStatement.
    def exitSelectionStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchStatement.
    def enterSwitchStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchStatement.
    def exitSwitchStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchBlock.
    def enterSwitchBlock(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchBlock.
    def exitSwitchBlock(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchSection.
    def enterSwitchSection(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchSection.
    def exitSwitchSection(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#switchLabel.
    def enterSwitchLabel(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#switchLabel.
    def exitSwitchLabel(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#iterationStatement.
    def enterIterationStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#iterationStatement.
    def exitIterationStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#whileStatement.
    def enterWhileStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#whileStatement.
    def exitWhileStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#doStatement.
    def enterDoStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#doStatement.
    def exitDoStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#forStatement.
    def enterForStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#forStatement.
    def exitForStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#forLoopInitializer.
    def enterForLoopInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#forLoopInitializer.
    def exitForLoopInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#forInStatement.
    def enterForInStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#forInStatement.
    def exitForInStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#jumpStatement.
    def enterJumpStatement(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#jumpStatement.
    def exitJumpStatement(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#expressions.
    def enterExpressions(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#expressions.
    def exitExpressions(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#expression.
    def enterExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#expression.
    def exitExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#assignmentOperator.
    def enterAssignmentOperator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#assignmentOperator.
    def exitAssignmentOperator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#castExpression.
    def enterCastExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#castExpression.
    def exitCastExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#initializer.
    def enterInitializer(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#initializer.
    def exitInitializer(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#constantExpression.
    def enterConstantExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#constantExpression.
    def exitConstantExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#unaryExpression.
    def enterUnaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#unaryExpression.
    def exitUnaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#unaryOperator.
    def enterUnaryOperator(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#unaryOperator.
    def exitUnaryOperator(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#postfixExpression.
    def enterPostfixExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#postfixExpression.
    def exitPostfixExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#postfix.
    def enterPostfix(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#postfix.
    def exitPostfix(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#argumentExpressionList.
    def enterArgumentExpressionList(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#argumentExpressionList.
    def exitArgumentExpressionList(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#argumentExpression.
    def enterArgumentExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#argumentExpression.
    def exitArgumentExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#primaryExpression.
    def enterPrimaryExpression(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#primaryExpression.
    def exitPrimaryExpression(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#constant.
    def enterConstant(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#constant.
    def exitConstant(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#stringLiteral.
    def enterStringLiteral(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#stringLiteral.
    def exitStringLiteral(self, ctx):
        pass


    # Enter a parse tree produced by ObjectiveCParser#identifier.
    def enterIdentifier(self, ctx):
        pass

    # Exit a parse tree produced by ObjectiveCParser#identifier.
    def exitIdentifier(self, ctx):
        pass


